@@ -34,9 +34,12 @@ class TestApp(cliff.app.App):
 class TestCase(unittest.TestCase):
 
     @mock.patch.object(ansible, "install_galaxy_roles", autospec=True)
+    @mock.patch.object(ansible, "passwords_yml_exists", autospec=True)
     @mock.patch.object(commands.KayobeAnsibleMixin,
                        "run_kayobe_playbooks")
-    def test_control_host_bootstrap(self, mock_run, mock_install):
+    def test_control_host_bootstrap(self, mock_run, mock_passwords,
+                                    mock_install):
+        mock_passwords.return_value = False
         command = commands.ControlHostBootstrap(TestApp(), [])
         parser = command.get_parser("test")
         parsed_args = parser.parse_args([])
@@ -44,12 +47,56 @@ class TestCase(unittest.TestCase):
         self.assertEqual(0, result)
         mock_install.assert_called_once_with(parsed_args)
         expected_calls = [
-            mock.call(mock.ANY, ["ansible/bootstrap.yml"]),
+            mock.call(mock.ANY, ["ansible/bootstrap.yml"], ignore_limit=True),
             mock.call(mock.ANY, ["ansible/kolla-ansible.yml"],
-                      tags="install"),
+                      tags="install", ignore_limit=True),
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
 
+    @mock.patch.object(ansible, "install_galaxy_roles", autospec=True)
+    @mock.patch.object(ansible, "passwords_yml_exists", autospec=True)
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbooks")
+    @mock.patch.object(commands.KollaAnsibleMixin,
+                       "run_kolla_ansible_overcloud")
+    def test_control_host_bootstrap_with_passwords(
+            self, mock_kolla_run, mock_run, mock_passwords, mock_install):
+        mock_passwords.return_value = True
+        command = commands.ControlHostBootstrap(TestApp(), [])
+        parser = command.get_parser("test")
+        parsed_args = parser.parse_args([])
+        result = command.run(parsed_args)
+        self.assertEqual(0, result)
+        mock_install.assert_called_once_with(parsed_args)
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                ["ansible/bootstrap.yml"],
+                ignore_limit=True
+            ),
+            mock.call(
+                mock.ANY,
+                ["ansible/kolla-ansible.yml"],
+                tags=None,
+                ignore_limit=True
+            ),
+            mock.call(
+                mock.ANY,
+                ["ansible/public-openrc.yml"],
+                ignore_limit=True
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run.call_args_list)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                "post-deploy",
+                extra_vars={"node_config_directory": "/etc/kolla"},
+            )
+        ]
+        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
+
     @mock.patch.object(ansible, "install_galaxy_roles", autospec=True)
     @mock.patch.object(ansible, "prune_galaxy_roles", autospec=True)
     @mock.patch.object(commands.KayobeAnsibleMixin,
@@ -63,9 +110,9 @@ class TestCase(unittest.TestCase):
         mock_install.assert_called_once_with(parsed_args, force=True)
         mock_prune.assert_called_once_with(parsed_args)
         expected_calls = [
-            mock.call(mock.ANY, ["ansible/bootstrap.yml"]),
+            mock.call(mock.ANY, ["ansible/bootstrap.yml"], ignore_limit=True),
             mock.call(mock.ANY, ["ansible/kolla-ansible.yml"],
-                      tags="install"),
+                      tags="install", ignore_limit=True),
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
 
@@ -194,6 +241,7 @@ class TestCase(unittest.TestCase):
                 mock.ANY,
                 ["ansible/kolla-ansible.yml"],
                 tags="config",
+                ignore_limit=True,
             ),
             mock.call(
                 mock.ANY,
@@ -525,10 +573,12 @@ class TestCase(unittest.TestCase):
                 mock.ANY,
                 ["ansible/kolla-ansible.yml"],
                 tags="config",
+                ignore_limit=True,
             ),
             mock.call(
                 mock.ANY,
                 ["ansible/kolla-bifrost.yml"],
+                ignore_limit=True,
             ),
             mock.call(
                 mock.ANY,
@@ -567,11 +617,16 @@ class TestCase(unittest.TestCase):
                 mock.ANY,
                 ["ansible/kolla-ansible.yml"],
                 tags="config",
+                ignore_limit=True,
+            ),
+            mock.call(
+                mock.ANY,
+                ["ansible/kolla-bifrost.yml"],
+                ignore_limit=True,
             ),
             mock.call(
                 mock.ANY,
                 [
-                    "ansible/kolla-bifrost.yml",
                     "ansible/seed-service-upgrade-prep.yml"
                 ],
             ),
@@ -595,6 +650,102 @@ class TestCase(unittest.TestCase):
         ]
         self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
 
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbooks")
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbook")
+    def test_overcloud_inventory_discover(self, mock_run_one, mock_run):
+        command = commands.OvercloudInventoryDiscover(TestApp(), [])
+        parser = command.get_parser("test")
+        parsed_args = parser.parse_args([])
+
+        result = command.run(parsed_args)
+        self.assertEqual(0, result)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                'ansible/overcloud-inventory-discover.yml',
+            ),
+            mock.call(
+                mock.ANY,
+                'ansible/ip-allocation.yml',
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run_one.call_args_list)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                ['ansible/kolla-ansible.yml'],
+                ignore_limit=True,
+                tags="config",
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run.call_args_list)
+
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbooks")
+    def test_overcloud_hardware_inspect(self, mock_run):
+        command = commands.OvercloudHardwareInspect(TestApp(), [])
+        parser = command.get_parser("test")
+        parsed_args = parser.parse_args([])
+
+        result = command.run(parsed_args)
+        self.assertEqual(0, result)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                [
+                    'ansible/kolla-bifrost-hostvars.yml',
+                    'ansible/overcloud-hardware-inspect.yml',
+                ],
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run.call_args_list)
+
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbooks")
+    def test_overcloud_provision(self, mock_run):
+        command = commands.OvercloudProvision(TestApp(), [])
+        parser = command.get_parser("test")
+        parsed_args = parser.parse_args([])
+
+        result = command.run(parsed_args)
+        self.assertEqual(0, result)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                [
+                    'ansible/kolla-bifrost-hostvars.yml',
+                    'ansible/overcloud-provision.yml',
+                ],
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run.call_args_list)
+
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbooks")
+    def test_overcloud_deprovision(self, mock_run):
+        command = commands.OvercloudDeprovision(TestApp(), [])
+        parser = command.get_parser("test")
+        parsed_args = parser.parse_args([])
+
+        result = command.run(parsed_args)
+        self.assertEqual(0, result)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                [
+                    'ansible/overcloud-deprovision.yml',
+                ],
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run.call_args_list)
+
     @mock.patch.object(commands.KayobeAnsibleMixin,
                        "run_kayobe_config_dump")
     @mock.patch.object(commands.KayobeAnsibleMixin,
@@ -644,6 +795,7 @@ class TestCase(unittest.TestCase):
                 mock.ANY,
                 ["ansible/kolla-ansible.yml"],
                 tags="config",
+                ignore_limit=True,
             ),
             mock.call(
                 mock.ANY,
