@@ -264,6 +264,35 @@ class TestCase(unittest.TestCase):
                                          quiet=False, env=expected_env)
         mock_vars.assert_called_once_with("/etc/kayobe")
 
+    @mock.patch.object(utils, "run_command")
+    @mock.patch.object(ansible, "_get_vars_files")
+    @mock.patch.object(ansible, "_validate_args")
+    def test_run_playbooks_ignore_limit(self, mock_validate, mock_vars,
+                                        mock_run):
+        mock_vars.return_value = ["/etc/kayobe/vars-file1.yml",
+                                  "/etc/kayobe/vars-file2.yaml"]
+        parser = argparse.ArgumentParser()
+        ansible.add_args(parser)
+        vault.add_args(parser)
+        args = [
+            "-l", "group1:host",
+        ]
+        parsed_args = parser.parse_args(args)
+        ansible.run_playbooks(parsed_args, ["playbook1.yml", "playbook2.yml"],
+                              limit="foo", ignore_limit=True)
+        expected_cmd = [
+            "ansible-playbook",
+            "--inventory", "/etc/kayobe/inventory",
+            "-e", "@/etc/kayobe/vars-file1.yml",
+            "-e", "@/etc/kayobe/vars-file2.yaml",
+            "playbook1.yml",
+            "playbook2.yml",
+        ]
+        expected_env = {"KAYOBE_CONFIG_PATH": "/etc/kayobe"}
+        mock_run.assert_called_once_with(expected_cmd, check_output=False,
+                                         quiet=False, env=expected_env)
+        mock_vars.assert_called_once_with("/etc/kayobe")
+
     @mock.patch.object(utils, "run_command")
     @mock.patch.object(ansible, "_get_vars_files")
     @mock.patch.object(ansible, "_validate_args")
@@ -408,3 +437,29 @@ class TestCase(unittest.TestCase):
         ]
         mock_remove.assert_called_once_with(expected_roles,
                                             "ansible/roles")
+
+    @mock.patch.object(utils, 'is_readable_file', autospec=True)
+    def test_passwords_yml_exists_false(self, mock_is_readable):
+        parser = argparse.ArgumentParser()
+        ansible.add_args(parser)
+        parsed_args = parser.parse_args([])
+        mock_is_readable.return_value = {"result": False}
+
+        result = ansible.passwords_yml_exists(parsed_args)
+
+        self.assertFalse(result)
+        mock_is_readable.assert_called_once_with(
+            "/etc/kayobe/kolla/passwords.yml")
+
+    @mock.patch.object(utils, 'is_readable_file', autospec=True)
+    def test_passwords_yml_exists_true(self, mock_is_readable):
+        parser = argparse.ArgumentParser()
+        ansible.add_args(parser)
+        parsed_args = parser.parse_args(["--config-path", "/path/to/config"])
+        mock_is_readable.return_value = {"result": True}
+
+        result = ansible.passwords_yml_exists(parsed_args)
+
+        self.assertTrue(result)
+        mock_is_readable.assert_called_once_with(
+            "/path/to/config/kolla/passwords.yml")
